@@ -31,6 +31,7 @@ from typing import Dict
 from typing import Pattern
 from typing import Match
 from typing import Callable
+from typing import NamedTuple
 from typing import Optional
 from typing import Type
 from typing import Any
@@ -105,16 +106,17 @@ class BaseTracker(BasePlugin):  # pylint: disable=too-many-instance-attributes
 
         assert self._COMMENT_REGEXP.pattern != self.__D_COMMENT_REGEXP.pattern
 
-        self._timeout = timeout
-        self._retries = retries
-        self._retries_sleep = retries_sleep
-        self._user_agent = user_agent
-        self._proxy_url = proxy_url
-        self._check_version = check_version
-        self._check_fingerprint = check_fingerprint
+        self.__timeout = timeout
+        self.__retries = retries
+        self.__retries_sleep = retries_sleep
+        self.__user_agent = user_agent
+        self.__proxy_url = proxy_url
+        self.__check_version = check_version
+        self.__check_fingerprint = check_fingerprint
 
-        self._cookie_jar: Optional[http.cookiejar.CookieJar] = None
-        self._opener: Optional[urllib.request.OpenerDirector] = None
+        self.__opener: Optional[urllib.request.OpenerDirector] = None
+
+        self.__cookie_jar: Optional[http.cookiejar.CookieJar] = None
 
     @classmethod
     def get_options(cls) -> Dict[str, Option]:
@@ -129,13 +131,13 @@ class BaseTracker(BasePlugin):  # pylint: disable=too-many-instance-attributes
         }
 
     def test(self) -> None:
-        if self._check_fingerprint or self._check_version:
-            opener = web.build_opener(self._proxy_url)
-            info = self._get_upstream_info(opener)
-        if self._check_fingerprint:
-            self._test_fingerprint(info["fingerprint"], opener)
-        if self._check_version:
-            self._test_version(info["version"])
+        if self.__check_fingerprint or self.__check_version:
+            opener = web.build_opener(self.__proxy_url)
+            info = self.__get_upstream_info(opener)
+        if self.__check_fingerprint:
+            self.__test_fingerprint(info["fingerprint"], opener)
+        if self.__check_version:
+            self.__test_version(info["version"])
 
     def is_matched_for(self, torrent: Torrent) -> bool:
         return (self._COMMENT_REGEXP.match(torrent.get_comment()) is not None)
@@ -143,7 +145,7 @@ class BaseTracker(BasePlugin):  # pylint: disable=too-many-instance-attributes
     def fetch_new_data(self, torrent: Torrent) -> bytes:
         raise NotImplementedError
 
-    # ===
+    # =====
 
     def _encode(self, arg: str) -> bytes:
         return arg.encode(self._SITE_ENCODING)
@@ -151,21 +153,50 @@ class BaseTracker(BasePlugin):  # pylint: disable=too-many-instance-attributes
     def _decode(self, arg: bytes) -> str:
         return arg.decode(self._SITE_ENCODING)
 
-    # ===
+    def _urlencode(self, arg: Dict) -> bytes:
+        return self._encode(urllib.parse.urlencode(arg))
+
+    # =====
 
     def _init_opener(self, with_cookies: bool) -> None:
+        assert not self.__opener
         if with_cookies:
-            self._cookie_jar = http.cookiejar.CookieJar()
-            self._opener = web.build_opener(self._proxy_url, self._cookie_jar)
+            self.__cookie_jar = http.cookiejar.CookieJar()
+            self.__opener = web.build_opener(self.__proxy_url, self.__cookie_jar)
         else:
-            self._opener = web.build_opener(self._proxy_url)
+            self.__opener = web.build_opener(self.__proxy_url)
 
-    def _build_opener(self) -> urllib.request.OpenerDirector:
-        return web.build_opener(self._proxy_url)
+    def _set_cookie(self, name: str, value: str, **kwargs: Any) -> None:
+        assert self.__cookie_jar
+        params = {
+            "version": 0,
+            "port": None,
+            "port_specified": False,
+            "domain": "",
+            "domain_specified": False,
+            "domain_initial_dot": False,
+            "path": "/",
+            "path_specified": True,
+            "secure": False,
+            "expires": None,
+            "discard": True,
+            "comment": None,
+            "comment_url": None,
+            "rest": {"HttpOnly": None},
+            "rfc2109": False,
+        }
+        params.update(kwargs)
+        self.__cookie_jar.set_cookie(http.cookiejar.Cookie(
+            name=name,
+            value=value,
+            **params,  # type: ignore
+        ))
 
     def _read_url(self, *args: Any, **kwargs: Any) -> bytes:
+        if not kwargs.get("opener"):
+            kwargs["opener"] = self.__opener
         try:
-            return self._read_url_nofe(*args, **kwargs)
+            return self.__read_url_nofe(*args, **kwargs)
         except (
             socket.timeout,
             urllib.error.HTTPError,
@@ -176,32 +207,30 @@ class BaseTracker(BasePlugin):  # pylint: disable=too-many-instance-attributes
         ) as err:
             raise NetworkError(err)
 
-    def _read_url_nofe(
+    def __read_url_nofe(
         self,
         url: str,
+        *,
         data: Optional[bytes]=None,
         headers: Optional[Dict[str, str]]=None,
         opener: urllib.request.OpenerDirector=None,
     ) -> bytes:
 
-        opener = (opener or self._opener)
-        assert opener is not None
-
+        assert opener
         headers = (headers or {})
-        headers.setdefault("User-Agent", self._user_agent)
-
+        headers.setdefault("User-Agent", self.__user_agent)
         return web.read_url(
             opener=opener,
             url=url,
             data=data,
             headers=headers,
-            timeout=self._timeout,
-            retries=self._retries,
-            retries_sleep=self._retries_sleep,
+            timeout=self.__timeout,
+            retries=self.__retries,
+            retries_sleep=self.__retries_sleep,
             retry_codes=self._SITE_RETRY_CODES,
         )
 
-    # ===
+    # =====
 
     def _assert_logic(self, arg: Any, *args: Any) -> None:
         _assert(LogicError, arg, *args)
@@ -228,12 +257,12 @@ class BaseTracker(BasePlugin):  # pylint: disable=too-many-instance-attributes
         self._assert_logic(is_valid_torrent_data(data), msg)
         return data
 
-    # ===
+    # =====
 
-    def _get_upstream_info(self, opener: urllib.request.OpenerDirector) -> Dict:
+    def __get_upstream_info(self, opener: urllib.request.OpenerDirector) -> Dict:
         try:
-            return json.loads(self._read_url_nofe(
-                url="https://raw.githubusercontent.com/mdevaev/emonoda/master/trackers/{}.json".format(self.PLUGIN_NAME),
+            return json.loads(self.__read_url_nofe(
+                url="https://raw.githubusercontent.com/mdevaev/emonoda/master/trackers/{}.json".format(self.PLUGIN_NAMES[0]),
                 opener=opener,
             ).decode("utf-8"))
         except urllib.error.HTTPError as err:
@@ -242,7 +271,7 @@ class BaseTracker(BasePlugin):  # pylint: disable=too-many-instance-attributes
             raise
 
     @classmethod
-    def _get_local_info(cls) -> Dict:
+    def _get_local_info(cls) -> Dict:  # Public for Makefile
         return {
             "version": cls._SITE_VERSION,
             "fingerprint": {
@@ -252,7 +281,7 @@ class BaseTracker(BasePlugin):  # pylint: disable=too-many-instance-attributes
             },
         }
 
-    def _test_fingerprint(self, fingerprint: Dict[str, str], opener: urllib.request.OpenerDirector) -> None:
+    def __test_fingerprint(self, fingerprint: Dict[str, str], opener: urllib.request.OpenerDirector) -> None:
         data = self._read_url(fingerprint["url"], opener=opener)
         msg = "Invalid site body, maybe tracker is blocked"
         try:
@@ -261,7 +290,7 @@ class BaseTracker(BasePlugin):  # pylint: disable=too-many-instance-attributes
             raise TrackerError(msg)
         _assert(TrackerError, fingerprint["text"] in page, msg)
 
-    def _test_version(self, upstream: int) -> None:
+    def __test_version(self, upstream: int) -> None:
         _assert(
             TrackerError,
             self._SITE_VERSION >= upstream,
@@ -289,7 +318,7 @@ class WithLogin(BaseTracker):
 
     def _login_using_post(self, url: str, post: Dict[str, bytes], ok_text: str) -> None:
         self._assert_required_user_passwd()
-        page = self._decode(self._read_url(url, data=self._encode(urllib.parse.urlencode(post))))
+        page = self._decode(self._read_url(url, data=self._urlencode(post)))
         self._assert_auth(ok_text in page, "Invalid user or password")
 
     def _assert_auth(self, *args: Any) -> None:
@@ -394,13 +423,13 @@ class WithCheckTime(BaseTracker):
 
     def _select_tzinfo(self, site_timezone: Optional[str]) -> datetime.tzinfo:
         if not site_timezone or self._default_timezone:
-            return self._get_default_tzinfo()
+            return self.__get_default_tzinfo()
         try:
             return pytz.timezone(site_timezone)
         except pytz.UnknownTimeZoneError:  # type: ignore
-            return self._get_default_tzinfo()
+            return self.__get_default_tzinfo()
 
-    def _get_default_tzinfo(self) -> datetime.tzinfo:
+    def __get_default_tzinfo(self) -> datetime.tzinfo:
         msg = "Can't determine timezone of site, your must configure it manually"
         self._assert_logic(bool(self._default_timezone), msg)
         return pytz.timezone(self._default_timezone)
@@ -448,6 +477,43 @@ class WithFetchByDownloadId(BaseTracker):
         ))
 
 
+class TrackerStat(NamedTuple):
+    seeders: int = 0
+    leechers: int = 0
+
+
+class WithStat(BaseTracker):  # pylint: disable=abstract-method
+    _STAT_URL = __D_STAT_URL = "{torrent_id}"
+    _STAT_OK_REGEXP = __D_STAT_OK_REGEXP = re.compile(r".*")
+    _STAT_SEEDERS_REGEXP = __D_STAT_SEEDERS_REGEXP = re.compile(r"(?P<seeders>.*)")
+    _STAT_LEECHERS_REGEXP = __D_STAT_LEECHERS_REGEXP = re.compile(r"(?P<leechers>.*)")
+
+    def __init__(self, **_: Any) -> None:  # pylint: disable=super-init-not-called
+        assert self._STAT_URL != self.__D_STAT_URL
+        assert self._STAT_OK_REGEXP.pattern != self.__D_STAT_OK_REGEXP.pattern
+        assert self._STAT_SEEDERS_REGEXP.pattern != self.__D_STAT_SEEDERS_REGEXP.pattern
+        assert self._STAT_LEECHERS_REGEXP.pattern != self.__D_STAT_LEECHERS_REGEXP.pattern
+
+    def fetch_stat(self, torrent: Torrent) -> TrackerStat:
+        torrent_id = self._assert_match(torrent)
+        page = self._decode(self._read_url(self._STAT_URL.format(torrent_id=torrent_id)))
+        self._assert_logic_re_search(
+            regexp=self._STAT_OK_REGEXP,
+            text=page,
+            msg="Missing OK mark",
+        )
+        return TrackerStat(
+            seeders=self.__parse_stat_int(page, self._STAT_SEEDERS_REGEXP, "seeders"),
+            leechers=self.__parse_stat_int(page, self._STAT_LEECHERS_REGEXP, "leechers"),
+        )
+
+    def __parse_stat_int(self, page: str, regexp: Pattern, group: str) -> int:
+        match = regexp.search(page)
+        if match:
+            return int(match.group(group))
+        return 0
+
+
 # =====
 def get_tracker_class(name: str) -> Type[BaseTracker]:
     return get_classes("trackers")[name]  # type: ignore
