@@ -24,7 +24,13 @@ from . import nnm_club_me
 
 # =====
 class Plugin(nnm_club_me.Plugin):
-    PLUGIN_NAME = _NNM_DOMAIN = "ipv6.nnm-club.name"
+    PLUGIN_NAMES = [
+        "ipv6.nnm-club.name",
+        "ipv6.nnm-club.me",
+        "ipv6.nnmclub.to",
+    ]
+
+    _NNM_DOMAIN = PLUGIN_NAMES[0]
 
     _SITE_VERSION = 1
     _SITE_FINGERPRINT_URL = "http://{}".format(_NNM_DOMAIN)
@@ -36,3 +42,5 @@ class Plugin(nnm_club_me.Plugin):
 
     _DOWNLOAD_ID_URL = "http://{}/forum/viewtopic.php?p={{torrent_id}}".format(_NNM_DOMAIN)
     _DOWNLOAD_URL = "http://{}//forum/download.php?id={{download_id}}".format(_NNM_DOMAIN)
+
+    _STAT_URL = _DOWNLOAD_ID_URL
